package controller;

import api.ResponseHandler;
import com.github.cliftonlabs.json_simple.JsonArray;
import com.github.cliftonlabs.json_simple.JsonObject;
import model.personnality.Trait;
import model.questionnaire.Answer;
import model.questionnaire.Question;
import model.questionnaire.Questionnaire;
import org.junit.jupiter.api.Test;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.*;

class SessionControllerTest {

    public String getSessionId(ResponseHandler iResponseHandler){
        //Get a valid session id
        SessionController vSessionController = new SessionController();
        JsonObject vSessionObject = iResponseHandler.getResponse("GET",new String[]{},"", vSessionController);
        assertTrue(vSessionObject.containsKey("id"));
        return (String) vSessionObject.get("id");
    }

    public float getTraitWeight(JsonObject iJsonObject)
    {
        return (float) ((JsonObject)((JsonArray)((JsonObject) iJsonObject.get("results")).get("traits")).get(0)).get("weight");
    }

    @Test
    public void TestSessionController(){
        ResponseHandler vHandler = new ResponseHandler();
        SessionController vController = new SessionController();
        String vSessionId = getSessionId(vHandler);

        Trait vTrait1 = new Trait("1","desc1");
        Trait vTrait2 = new Trait("2","desc2");

        Answer vAnswer0 = new Answer("1",vTrait1,0);
        Answer vAnswerMax = new Answer("2",vTrait1,Float.MAX_VALUE);
        Answer vAnswerMin = new Answer("3",vTrait1,Float.MIN_VALUE);

        Question vQuestion1 = new Question("1", Arrays.asList(
                vAnswer0,
                vAnswerMax,
                vAnswerMin
        ));
        Question vQuestion2 = new Question("2", Arrays.asList(
                vAnswer0,
                vAnswerMax,
                vAnswerMin
        ));

        Questionnaire vQuestionnaire = new Questionnaire("1",Arrays.asList(
                vQuestion1,vQuestion2
        ));
        //Verify update session question answer
        JsonObject vObject = new JsonObject().putChain("question",vQuestion1.getId().toString()).putChain("answer",vAnswerMax.getId().toString());
        try {
            vController.updateEntry(vSessionId,vObject);
        } catch (Exception e) {
            assertNotEquals(false,true);
            e.printStackTrace();
        }
        assertEquals(Float.MAX_VALUE,getTraitWeight(vController.getEntriesForId(vSessionId,null)));
        try {
            vController.updateEntry(vSessionId,vObject);
        } catch (Exception e) {
            assertNotEquals(false,true);
            e.printStackTrace();
        }
        assertEquals(Float.MAX_VALUE,getTraitWeight(vController.getEntriesForId(vSessionId,null)));

        vObject.putChain("answer", vAnswerMin.getId().toString());

        try {
            vController.updateEntry(vSessionId,vObject);
        } catch (Exception e) {
            assertNotEquals(false,true);
            e.printStackTrace();
        }
        assertEquals(Float.MIN_VALUE,getTraitWeight(vController.getEntriesForId(vSessionId,null)));

        try {
            vController.updateEntry(vSessionId,vObject);
        } catch (Exception e) {
            assertNotEquals(false,true);
            e.printStackTrace();
        }
        assertEquals(Float.MIN_VALUE,getTraitWeight(vController.getEntriesForId(vSessionId,null)));

        vObject.putChain("answer",1);
        assertThrows(Exception.class, () -> vController.updateEntry(vSessionId,vObject));
        assertEquals(Float.MIN_VALUE,getTraitWeight(vController.getEntriesForId(vSessionId,null)));

        vObject.putChain("answer","12");
        assertThrows(Exception.class, () -> vController.updateEntry(vSessionId,vObject));
        assertEquals(Float.MIN_VALUE,getTraitWeight(vController.getEntriesForId(vSessionId,null)));

        vObject.putChain("answer", UUID.randomUUID().toString());
        assertThrows(Exception.class, () -> vController.updateEntry(vSessionId,vObject));
        assertEquals(Float.MIN_VALUE,getTraitWeight(vController.getEntriesForId(vSessionId,null)));

    }
}